@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2003-2008)
+#   Copyright (C) Dr. Heinz-Josef Claes (2003-2009)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 #
 
 
-my $VERSION = '$Id: storeBackupDel.pl 347 2008-11-04 15:29:59Z hjc $ ';
+my $VERSION = '$Id: storeBackupDel.pl 354 2009-03-15 08:39:18Z hjc $ ';
 push @VERSION, $VERSION;
 my ($VERSIONpName, $VERSIONsvnID) = $VERSION =~ /Id:\s+(\S+)\s+(\d+)/;
 $main::STOREBACKUPVERSION = undef;
@@ -112,7 +112,7 @@ this program deletes backups created by storeBackup
 
 usage:
 	$prog [-f configFile] [--print]
-	[--b backupDirectory] [-S series] [--doNotDelete]
+	[-b backupDirectory] [-S series] [--doNotDelete]
 	[--deleteNotFinishedDirs] [-L lockFile]
 	[--keepAll timePeriod] [--keepWeekday entry] [--keepFirstOfYear]
 	[--keepLastOfYear] [--keepFirstOfMonth] [--keepLastOfMonth]
@@ -128,7 +128,7 @@ usage:
 --backupDir	-b  top level directory of all backups (must exist)
 --series	-S  directory of backup series
 		    same parameter as in storeBackup / relative path
-		    from backupDir
+		    from backupDir, default is 'default'
 --lockFile      -L  lock file, if exists, new instances will finish if
 		    an old is allready running, default is $lockFile
 --doNotDelete	    test only, do not delete any backup
@@ -506,7 +506,6 @@ if ($todayOpt)
 	$prLog->print('-kind' => 'W',
 		      '-str' => ["setting today to " .
 				 $today->getDateTime()]);
-
     }
     else
     {
