@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2002-2008)
+#   Copyright (C) Dr. Heinz-Josef Claes (2002-2009)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 #
 
 
-my $VERSION = '$Id: storeBackupRecover.pl 349 2008-11-08 09:39:55Z hjc $ ';
+my $VERSION = '$Id: storeBackupRecover.pl 354 2009-03-15 08:39:18Z hjc $ ';
 push @VERSION, $VERSION;
 my ($VERSIONpName, $VERSIONsvnID) = $VERSION =~ /Id:\s+(\S+)\s+(\d+)/;
 $main::STOREBACKUPVERSION = undef;
@@ -311,6 +311,21 @@ my $gnucp = ($$meta{'cpIsGnu'})->[0];
 $gnucp = ($gnucp eq 'yes') ? 1 : 0;
 $gnucp = 0 if $noGnuCp;
 
+$main::IOCompressDirect = 0;
+if ($uncompr eq 'bzip2' or $uncompr eq 'bunzip2')
+{
+    eval "use IO::Uncompress::Bunzip2 qw(bunzip2)";
+    if ($@)
+    {
+	$prLog->print('-kind' => 'I',
+		      '-str' => ["please install IO::Uncompress::Bunzip2 from " .
+				 "CPAN for better performance"]);
+    }
+    else
+    {
+	$main::IOCompressDirect = 1;
+    }
+}
 
 # dbm-File ffnen
 my %DBMHardLink;        # key: dev-inode (oder undef), value: filename
@@ -323,6 +338,7 @@ unless ($noHardLinks)
 
 my $noFilesCopy = 0;
 my $noFilesCompr = 0;
+my $noFilesBlocked = 0;
 my $noSymLinks = 0;
 my $noNamedPipes = 0;
 my $noSockets = 0;
@@ -389,6 +405,35 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 	}
 	elsif ($md5sum eq 'symlink')
 	{
+	    unless ($noHardLinks)
+	    {
+		if (exists($DBMHardLink{$devInode}))   # muss nur gelinkt werden
+		{
+		    if (link $DBMHardLink{$devInode}, $targetFile)
+		    {
+			$prLog->print('-kind' => 'I',
+				      '-str' =>
+				      ["link " . $DBMHardLink{$devInode} .
+				       " $targetFile"])
+			    if $verbose;
+#			utime $atime, $mtime, $f;
+			++$hardLinks;
+		    }
+		    else
+		    {
+			$prLog->print('-kind' => 'E',
+				      '-str' =>
+				      ["failed: link " .
+				       $DBMHardLink{$devInode} .
+				       " $targetFile"]);
+		    }
+		    goto contLoop;
+		}
+		else
+		{
+		    $DBMHardLink{$devInode} = $targetFile;
+		}
+	    }
 	    my $linkTo = readlink "$backupRoot/$filename";
 	    if (not $overwrite and -e $targetFile)
 	    {
@@ -413,7 +458,7 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 		$chown->wait();
 #		utime $atime, $mtime, $targetFile;
 		$prLog->print('-kind' => 'I',
-			      '-str' => ["ln -s $linkTo $targetFile\n"])
+			      '-str' => ["ln -s $linkTo $targetFile"])
 		    if ($verbose);
 	    }
 	}
@@ -499,7 +544,7 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 			if ($verbose);
 		}
 	    }
-	    else                   # war komprimiert
+	    elsif ($compr eq 'c')          # war komprimiert
 	    {
 		if (not $overwrite and -e $targetFile)
 		{
@@ -515,7 +560,7 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 		    ++$noFilesCompr;
 		    ($old, $new) =
 			$fork->add_block('-exec' => $uncompr,
-				   '-param' => [@uncomprPar],
+				   '-param' => \@uncomprPar,
 				   '-stdin' => "$backupRoot/$filename$postfix",
 				   '-stdout' => "$targetFile",
 				   '-delStdout' => 'no',
@@ -529,7 +574,29 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 			if ($verbose);
 		}
 	    }
-
+	    elsif ($compr eq 'b')       # blocked file
+	    {
+		++$noFilesBlocked;
+		($old, $new) =
+		    $fork->add_block('-function' => \&uncompressCatBlock,
+				     '-funcPar' => ["$backupRoot/$filename",
+				     $targetFile, '\A\d.*', $uncompr, \@uncomprPar,
+				     $postfix, $prLog],
+				     '-info' => [$targetFile, $uid, $gid, $mode,
+						 $atime, $mtime, $devInode]);
+		$prLog->print('-kind' => 'I',
+			      '-str' => ["cp (blocked) " .
+					     "$backupRoot/$filename$postfix " .
+					     "$targetFile"])
+			if ($verbose);
+	    }
+	    else
+	    {
+		$prLog->print('-kind' => 'E',
+			      '-str' =>
+			      ["unknow compr flag <$compr> in .md5CheckSums " .
+			       "for file <$backupRoot/$filename>"]);
+	    }
 	    if ($old)
 	    {
 		my ($f, $oUid, $oGid, $oMode, $oAtime, $oMtime, $oDevInode) =
@@ -595,7 +662,9 @@ $prLog->print('-kind' => 'I',
 		    ($noBlockDevs ? "$noBlockDevs block devs" : ()),
 		    ($noCharDevs ? "$noCharDevs char devs" : ()),
 		    ($noFilesCopy ? "$noFilesCopy copied" : ()),
-		    ($noFilesCompr ? "$noFilesCompr uncompressed" : ()))])
+		    ($noFilesCompr ? "$noFilesCompr uncompressed" : ()),
+		    ($noFilesBlocked ? "$noFilesBlocked cat blocked files" : ()))]
+    )
     if ($noRestored);
 
 exit 0;
@@ -619,24 +688,100 @@ sub setHardLinks
 			      '-str' => ["cannot link <$f> to itself"]);
 	    }
 	    else
-	    {
-		
+	    {		
 		if (link $oldF, $f)
 		{
 		    $prLog->print('-kind' => 'I',
 				  '-str' => ["link $oldF $f"])
 			if ($verbose);
+		    chown $uid, $gid, $f;
+		    chmod $mode, $f;
+		    utime $atime, $mtime, $f;
 		}
 		else
 		{
 		    $prLog->print('-kind' => 'E',
 				  '-str' => ["failed: link $oldF $f"]);
 		}
-		chown $uid, $gid, $f;
-		chmod $mode, $f;
-		utime $atime, $mtime, $f;
 	    }
 	    delete $$hasToBeLinked{$f};
 	}
     }
 }
+
+
+########################################
+sub uncompressCatBlock
+{
+    my $fromDir = shift;
+    my $toFile = shift;
+    my $mask = shift;
+    my $umcompr = shift;
+    my $uncomprPar = shift;
+    my $postfix = shift;
+    my $prLog = shift;
+
+    local *DIR;
+    unless (opendir(DIR, $fromDir))
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot open <$fromDir>"]);
+	return 1;
+    }
+    my ($entry, @entries);
+    while ($entry = readdir DIR)
+    {
+	next unless $entry =~ /$mask/;
+
+	push @entries, $entry;
+    }
+    close(DIR);
+
+    local *TO;
+    unless (sysopen(TO, $toFile, O_CREAT | O_WRONLY))
+    {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot write to <$toFile>"]);
+    }
+
+    foreach $entry (sort @entries)
+    {
+	local *FROM;
+	if ($entry =~ /$postfix\Z/)    # compressed block
+	{
+	    if ($main::IOCompressDirect)
+	    {
+		my $input = "$fromDir/$entry";
+		my $buffer;
+		my $uc = new IO::Uncompress::Bunzip2 $input;
+		$uc->read($buffer);
+		syswrite(TO, $buffer);
+		next;
+	    }
+
+	    unless (open(FROM, "$uncompr @$uncomprPar < \'$fromDir/$entry\' |"))
+	    {
+		$prLog->print('-kind' => 'E',
+			      '-str' => ["cannot read <$fromDir/$entry>"]);
+		return 1;
+	    }
+	}
+	else           # block not compressed
+	{
+	    unless (sysopen(FROM, "$fromDir/$entry", O_RDONLY))
+	    {
+		$prLog->print('-kind' => 'E',
+			      '-str' => ["cannot read <$fromDir/$entry>"]);
+		return 1;
+	    }
+	}
+	my $buffer;
+	while (sysread(FROM, $buffer, 10*1024**2))
+	{
+	    syswrite(TO, $buffer);
+	}
+	close(FROM);
+    }
+    close(TO);
+    return 0;
+}
