@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2008)
+#   Copyright (C) Dr. Heinz-Josef Claes (2008-2009)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -109,7 +109,7 @@ You need some basic understanding of linux and perl to use it.
 usage:
 	$prog -g configFile
 
-	$prog [-f configFile] [-b backupDir]
+	$prog [-f configFile] [-b backupDirDir]
 	      [-s rule]  [--absPath] [-w file] [--parJobs number]
 	      [-d level] [--once] [--print] [backupRoot . . .]
 
@@ -158,7 +158,7 @@ my $templateConfigFile = <<EOC;
 # a shell. Single quotes will mask environment variables, while double
 # quotes will not.
 # You can mask \$, {, }, ", ' with a backslash (\\), eg. \\\$
-# Lines beginning with a '#' are ignored (use this for comments)
+# Lines beginning with a '#' or ';' are ignored (use this for comments)
 #
 # You can overwrite settings in the command line. You can remove
 # the setting also in the command by using the --unset feature, eg.:
@@ -167,10 +167,7 @@ my $templateConfigFile = <<EOC;
 
 # *** param must exist ***
 # top level directory of all linked backups
-backupDir=
-
-# root directory of backup relative to backupDir directory
-#backupRoot=
+;backupDir=
 
 # *** param must exist ***
 # rule for searching
@@ -178,24 +175,27 @@ backupDir=
 # EXAMPLE: 
 # searchRule = ( '\$size > &::SIZE("3M")' and '\$uid eq "hjc"' ) or
 #    ( '\$mtime > &::DATE("3d4h")' and not '\$file =~ m#/tmp/#' )'
-searchRule=
+;searchRule=
+
+# root directory of backup relative to backupDir directory
+;backupRoot=
 
 # write result with absolute path names
 # default is 'no', possible values are 'yes' and 'no'
-#absPath
+;absPath
 
 # write search result also to file
-#writeToFile=
+;writeToFile=
 
 # number of parallel jobs, default = choosen automaticly
-#parJobs=
+;parJobs=
 
 # debug level, possible values are 0, 1, 2, default = 0
-#debug=
+;debug=
 
 # show every found file only once (depending on md5 sum)
 # default is 'no', possible values are 'yes' and 'no'
-#once =
+;once=
 EOC
     ;
 
@@ -606,7 +606,7 @@ sub checkRule
     {
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["cannot open temporary file <$tmpfile>"]);
-	exit 1;                         # ERROR
+	return 1;                         # ERROR
     }
 
     my ($l);
@@ -639,5 +639,5 @@ sub checkRule
 
     close(OUT);
 
-    exit 0;
+    return 0;
 }
