@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2008)
+#   Copyright (C) Dr. Heinz-Josef Claes (2008-2009)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -107,6 +107,7 @@ require 'humanRead.pl';
 
 my $lockFile = '/tmp/storeBackupUpdateBackup.lock';   # default value
 my $checkSumFile = '.md5CheckSums';
+my $blockCheckSumFile = '.md5BlockCheckSums';
 
 my $Help = <<EOH;
 this program updates / finalizes backups created by storeBackup.pl
@@ -553,6 +554,10 @@ unlink $lockFile;
 $prLog->print('-kind' => 'Z',
 	      '-str' => ["checking refernces in <@backupDirs>"]);
 
+$prLog->print('-kind' => 'I',
+	      '-str' => ["syncing ..."]);
+system "/bin/sync";
+
 exit 0;
 
 
@@ -610,7 +615,7 @@ sub updateBackupDir
     my ($oldSize, $newSize) = (0, 0);
     local *LINKFILE;
 
-    open(LINKFILE, "bzip2 -d < \"$f\" |") or
+    open(LINKFILE, "bzip2 -d < \'$f\' |") or
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["cannot open file <$f>"],
 		      '-exit' => 1);
@@ -634,17 +639,16 @@ sub updateBackupDir
 	    }
 	    $noMkdir++;
 	}
-	elsif ($what eq 'link')
+	elsif ($what eq 'link' or $what eq 'linkblock' or
+	    $what eq 'linkSymlink')
 	{
 	    my $existingFile = <LINKFILE>;
-	    $existingFile =~ s/\0/\n/og;
 	    $existingFile = "$dir/$existingFile";
 	    $prLog->print('-kind' => 'E',
 			  '-str' =>
 			  ["file <$f> ends unexpected at line $."],
 			  '-exit' => 1)
 		unless $existingFile;
-	    chomp $existingFile;
 
 	    my $newLink = <LINKFILE>;
 	    $prLog->print('-kind' => 'W',
@@ -683,7 +687,8 @@ sub updateBackupDir
 
 	    $file =~ s/\0/\n/og;
 	    $file = "$dir/$file";        # file to compress
-	    $oldSize += (stat($file))[7];
+	    my $st = (stat($file))[7];
+	    $oldSize += $st if defined $st;
 	    my ($old, $new) =
 		$parForkProc->add_block('-exec' => $compr,
 					'-param' => \@comprPar,
@@ -756,13 +761,24 @@ sub updateBackupDir
     $prLog->print('-kind' => 'I',
 		  '-str' => ["phase 2: setting hard links"]);
 
+    my $withBlockedFiles = 0;
     my $noHardLinks = 0;
     my $noCopiedFiles = 0;
-    my $pr = $progressReport * 1000;
-    open(LINKFILE, "bzip2 -d < \"$f\" |") or
+    my $pr = $progressReport * 500;
+    open(LINKFILE, "bzip2 -d < \'$f\' |") or
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["cannot open file <$f>"],
 		      '-exit' => 1);
+    local *BLOCKMD5;
+    if (-e "$dir/.md5BlockCheckSums")
+    {
+	$withBlockedFiles = 1;
+	open(BLOCKMD5, ">> $dir/.md5BlockCheckSums") or
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+			  ["cannot append to <$dir/.md5BlockCheckSums>"],
+			  '-exit' => 1);
+    }
 
     while ($l = <LINKFILE>)
     {
@@ -802,14 +818,17 @@ sub updateBackupDir
                 unless (::copy("$existingFile", "$newLink"))
                 {
                     $prLog->print('-kind' => 'E',
-                                  '-str' => ["could not copy $existingFile " .
-                                             "$newLink"]);
+                                  '-str' => ["could not link/copy " .
+					     "$existingFile $newLink"]);
                     next;
                 }
 		$noCopiedFiles++;
 		$md5ToFile{$md5} = $newLink;
 	    }
 	}
+	elsif ($what eq 'dir')
+	{
+	}
 	elsif ($what eq 'compress' or $what eq 'symlink')
 	{
 	    my $file = <LINKFILE>;
@@ -818,8 +837,77 @@ sub updateBackupDir
 			  '-exit' => 1)
 		unless $file;
 	}
-	elsif ($what eq 'dir')
+	elsif ($what eq 'linkSymlink')
 	{
+	    my $existingFile = <LINKFILE>;
+	    $existingFile = "$dir/$existingFile";
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["file <$f> ends unexpected at line $."],
+			  '-exit' => 1)
+		unless $existingFile;
+	    chomp $existingFile;
+	    $existingFile =~ s/\0/\n/og;
+
+	    my $newLink = <LINKFILE>;
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["file <$f> ends unexpected at line $."],
+			  '-exit' => 1)
+		unless $newLink;
+	    chomp $newLink;
+	    $newLink =~ s/\0/\n/og;
+	    $newLink = "$dir/$newLink";
+	    if (link $existingFile, $newLink)
+	    {
+		$noHardLinks++;
+		$prLog->print('-kind' => 'S',
+			      '-str' => ["linked $noHardLinks files"])
+			if ($pr and $noHardLinks % $pr == 0);
+	    }
+	    else
+	    {
+		# create symlink
+		$prLog->print('-kind' => 'E',
+			      '-str' =>
+			      ["cannot create symlink from <$newLink> -> <$md5>"])
+		unless symlink $md5, $newLink;
+	    $noSymLink++;
+	    }
+	}
+	elsif ($what eq 'linkblock')
+	{
+	    my $existingFile = <LINKFILE>;
+	    $existingFile = "$dir/$existingFile";
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["file <$f> ends unexpected at line $."],
+			  '-exit' => 1)
+		unless $existingFile;
+	    chomp $existingFile;
+	    $existingFile =~ s/\0/\n/og;
+
+	    my $newLink = <LINKFILE>;
+	    $newLink = "$dir/$newLink";
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["file <$f> ends unexpected at line $."],
+			  '-exit' => 1)
+		unless $newLink;
+	    chomp $newLink;
+	    $newLink =~ s/\0/\n/og;
+
+	    $noHardLinks +=
+		&::hardLinkDir($existingFile, $newLink, '\A\d.*', $prLog);
+
+	    local *BLOCKLOCAL;        # copy md5 sums to global dictionary
+	    open(BLOCKLOCAL, "bzip2 -d < \'$newLink/.md5BlockCheckSums.bz2\' |") or
+		$prLog->print('-kind' => 'E',
+			      '-str' =>
+			      ["cannot open <$newLink/.md5BlockCheckSums.bz2>"],
+			      '-exit' => 1);
+	    my $l;
+	    while ($l = <BLOCKLOCAL>)
+	    {
+		print BLOCKMD5 $l;
+	    }
+	    close(BLOCKLOCAL);
 	}
 	else
 	{
@@ -831,6 +919,14 @@ sub updateBackupDir
 	}
 
     }
+    if ($withBlockedFiles)
+    {
+	close(BLOCKMD5) or
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+			  ["cannot close file <$dir/.md5BlockCheckSums>"],
+			  '-exit' => 1);
+    }
     close(LINKFILE) or
 	$prLog->print('-kind' => 'E',
 		      '-str' => ["cannot close file <$f>"],
@@ -841,13 +937,24 @@ sub updateBackupDir
 		  '-str' => ["copied $noCopiedFiles files"])
 	if $noCopiedFiles;
 
+    my $comprMd5BlockCheckSums;
+    if ($withBlockedFiles and -e "$dir/.md5CheckSums.bz2")
+    {
+	# compress .md5BlockCheckSums
+	$comprMd5BlockCheckSums =
+	    forkProc->new('-exec' => 'bzip2',
+			  '-param' => ["$dir/.md5BlockCheckSums"],
+			  '-outRandom' =>
+			  '/tmp/stbu-compr-',
+			  '-prLog' => $prLog);
+    }
 
     #
     # set file permissions
     #
     my $preservePerms =
 	(($$meta{'preservePerms'})->[0] eq 'no') ? 0 : 1;
-    $pr = $progressReport * 10000;
+    $pr = $progressReport * 5000;
     if ($preservePerms)
     {
 	$prLog->print('-kind' => 'I',
@@ -882,6 +989,11 @@ sub updateBackupDir
 
 	    utime $atime, $mtime, $file;
 	    chown $uid, $gid, $file;
+	    if ($compr eq 'b')         # block file
+	    {
+		$mode &= 0777;    # strip special permissions
+		$mode |= 0111;    # add directory permissions
+	    }
 	    chmod $mode, $file;
 	}
 
@@ -895,6 +1007,19 @@ sub updateBackupDir
 				 "preservePerms not set in storeBackup.pl"]);
     }
 
+    if ($withBlockedFiles and -e "$dir/.md5CheckSums.bz2")
+    {
+	# compress .md5BlockCheckSums
+	$comprMd5BlockCheckSums->wait();
+	my $out = $comprMd5BlockCheckSums->getSTDOUT();
+	$prLog->print('-kind' => 'W',
+		      '-str' => ["STDERR of <uname>:", @$out])
+	    if (@$out > 0);
+	$out = $comprMd5BlockCheckSums->getSTDERR();
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["STDERR of <uname>:", @$out])
+	    if (@$out > 0);
+    }
 
     #
     # set directory permissions
