@@ -1,7 +1,7 @@
 #! /usr/bin/env perl
 
 #
-#   Copyright (C) Dr. Heinz-Josef Claes (2002-2008)
+#   Copyright (C) Dr. Heinz-Josef Claes (2002-2009)
 #                 hjclaes@web.de
 #   
 #   This program is free software: you can redistribute it and/or modify
@@ -19,11 +19,15 @@
 #
 
 
-my $VERSION = '$Id: storeBackupVersions.pl 347 2008-11-04 15:29:59Z hjc $ ';
+my $VERSION = '$Id: storeBackupVersions.pl 354 2009-03-15 08:39:18Z hjc $ ';
 push @VERSION, $VERSION;
 my ($VERSIONpName, $VERSIONsvnID) = $VERSION =~ /Id:\s+(\S+)\s+(\d+)/;
 $main::STOREBACKUPVERSION = undef;
 
+use Fcntl qw(O_RDWR O_CREAT);
+use File::Copy;
+use POSIX;
+use Digest::MD5 qw(md5_hex);
 
 use strict;
 
@@ -123,6 +127,9 @@ usage:
 --mtime		-m  show modify time of source file
 --atime		-a  show access time of source file
 
+It does not always work correctly when a file is saved blocked *and*
+non-blocked in different backups. In such cases, use option --locateSame.
+
 Copyright (c) 2002-2008 by Heinz-Josef Claes (see README)
 Published under the GNU General Public License v3 or any later version
 EOH
@@ -224,14 +231,14 @@ $prLog->print('-kind' => 'V',
 #
 # md5CheckSum - Datei finden
 $prLog->print('-kind' => 'E',
-	      '-str' => ["file <$f> does not exit"],
+	      '-str' => ["file <$f> does not exist"],
 	      '-exit' => 1)
-    unless (-f $f);
+    unless (-f $f or -d $f);
 
 if ($backupRoot)
 {
     $prLog->print('-kind' => 'E',
-		  '-str' => ["directory <$backupRoot> does not exit"],
+		  '-str' => ["directory <$backupRoot> does not exist"],
 		  '-exit' => 1)
 	unless (-d $backupRoot);
     $backupRoot = &absolutePath($backupRoot);
@@ -328,9 +335,10 @@ $prLog->print('-kind' => 'W',
     if @d;
 
 
-# Zuerst die Dateien direkt aus Existenz berprfen
+# Zuerst die Dateien direkt auf Existenz berprfen
 # dann md5-Summen berechnen, um unterschiedliche Stnde festzustellen
 my (@files, @md5sum, @dirs, $entry);
+my $md5sumBlock = undef;
 my $lastInode = undef;
 foreach $entry (@allDirs)
 {
@@ -357,27 +365,29 @@ foreach $entry (@allDirs)
 	$lastInode = $inode;
 
 	# md5 Summe muss berechnet werden
-	my $m = forkProc->new('-exec' => 'md5sum',
-			      '-param' => [$entry . '/' . $fileWithRelPath],
-			      '-prLog' => $prLog,
-			      '-workingDir' => '.',
-			      '-outRandom' => '/tmp/md5-');
-	$m->wait();
-	my $out = $m->getSTDERR();
+	my $md5 = &::calcFileMD5Sum("$f");
 	$prLog->print('-kind' => 'E',
-		      '-str' =>
-		      ["fork of md5sum generated the following errors:",
-		       @$out])
-	    if (@$out > 0);
-	my $out = $m->getSTDOUT();
-	my $l = $$out[0];
-	if ($l =~ /\A\\/)  # "\\" am Zeilenanfang -> es wird gequotet
+		      '-str' => ["could not read <$f>"])
+	    unless $md5;
+
+	push @md5sum, $md5;
+    }
+    elsif (-d $f)       # blocked file
+    {
+	# md5 sum of block check sum file
+	my $fBlock = "$f/.md5BlockCheckSums.bz2";
+	unless (-f $fBlock)
 	{
-	    $l =~ s/\\n/\n/g;   # "\n" im Namen wird von md5sum zu
-                                # "\\n" gemacht, zurckkonvertieren!
-	    $l =~ s/\A\\//;     # "\\" am Zeilenende entfernen
+	    $prLog->print('-kind' => 'E',
+			  '-str' => ["cannot open <$fBlock>"]);
+	    next;
 	}
-	my ($md5, $name) = $l =~ /\A(\w+)\s+(.*)/s;
+	my $md5 = &::calcFileMD5Sum("$fBlock");
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["could not read <$fBlock>"])
+	    unless $md5;
+	push @files, $f;
+	push @dirs, $entry;
 	push @md5sum, $md5;
     }
 }
