@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2008)
+#   Copyright (C) Heinz-Josef Claes (2001-2009)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 
 
 
-push @VERSION, '$Id: fileDir.pl 348 2008-11-04 19:33:20Z hjc $ ';
+push @VERSION, '$Id: fileDir.pl 354 2009-03-15 08:39:18Z hjc $ ';
 
 require 'prLog.pl';
 require 'forkProc.pl';
@@ -784,9 +784,11 @@ sub new
 
     $self->{'dirs'} = 0;         # hier wurde ein Directory gelscht
     $self->{'files'} = 0;        # hier wurde eine Datei gelscht
-    $self->{'bytes'} = 0;        # hier wurde eine Datei gelscht
+    $self->{'bytes'} = 0;        # hier wurde eine Datei gelscht,
+                                 # Datei hatte entsprechend bytes
     $self->{'links'} = 0;        # hier wurde nur ein Link weggenommen
     $self->{'stayBytes'} = 0;    # hier wurde nur ein Link weggenommen
+                                 # Anzahl Bytes bleiben bestehen
     my $dir = $params{'-dir'};
 
     my $ret = bless $self, $class;
