@@ -1,7 +1,7 @@
 # -*- perl -*-
 
 #
-#   Copyright (C) Heinz-Josef Claes (2001-2008)
+#   Copyright (C) Heinz-Josef Claes (2001-2009)
 #                 hjclaes@web.de
 #
 #   This program is free software: you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 
 
 
-push @VERSION, '$Id: forkProc.pl 347 2008-11-04 15:29:59Z hjc $ ';
+push @VERSION, '$Id: forkProc.pl 354 2009-03-15 08:39:18Z hjc $ ';
 
 use Carp;
 use strict;
@@ -54,7 +54,8 @@ sub new
     unless ($pid)    # we are now in the client
     {
 	my $function = $params{'-function'};
-	exit &$function(@{$params{'-funcPar'}});
+	my $ret = &$function(@{$params{'-funcPar'}});
+	exec "/bin/sh -c \"exit $ret\"";
     }
 
     $self->{'pid'} = $pid;      # we are now in the parent
