@@ -19,7 +19,7 @@
 #
 
 
-push @VERSION, '$Id: prLog.pl 347 2008-11-04 15:29:59Z hjc $ ';
+push @VERSION, '$Id: prLog.pl 350 2008-12-13 15:50:19Z hjc $ ';
 
 
 use IO::Handle;
@@ -122,6 +122,46 @@ sub new
 }
 
 
+##################################################
+sub setStopAtNoMessages
+{
+    my $self = shift;
+
+    # Defaultwerte f"ur Parameter setzen
+    my (%params) = ('-kind' => undef,
+		    '-stopAt' => undef,
+		    '-message' => 'too many errors, exiting',
+		    '-exit' => 1);
+
+
+    &::checkObjectParams(\%params, \@_, 'printLog::setStopAtNoMessages',
+			 ['-kind', '-stopAt']);
+
+    $self->{'encounteredStop'}->{$params{'-kind'}} = $params{'-stopAt'};
+    $self->{'encounteredKindStop'}->{$params{'-kind'}} =
+	$params{'-kind'};
+    $self->{'encounteredMessageStop'}->{$params{'-kind'}} =
+	$params{'-message'};
+    $self->{'encounteredExit'}->{$params{'-kind'}} =
+	$params{'-exit'};
+}
+
+
+##################################################
+sub encountered
+{
+    my $self = shift;
+
+    # Defaultwerte f"ur Parameter setzen
+    my (%params) = ('-kind' => undef);
+
+    &::checkObjectParams(\%params, \@_, 'printLog::encountered', ['-kind']);
+
+    my $ret = $self->{'encountered'}->{$params{'-kind'}};
+    return $ret ? $ret : 0;
+}
+
+
 ##################################################
 sub setFileSpecs
 {
@@ -170,6 +210,7 @@ sub print
     }
 
     my $kind = $params{'-kind'};
+    $self->{'encountered'}->{$kind} += 1;
 
     my $k = $self->{'kindhash'}{$kind};
     $k = $self->{'kindhash'}{'?'} unless ($k);
@@ -192,6 +233,19 @@ sub print
     }
     $self->__print(@a);
 
+    if (defined($self->{'encounteredStop'}->{$kind}) and
+	$self->{'encountered'}->{$kind} >=
+	$self->{'encounteredStop'}->{$kind})
+    {
+	$k = $self->{'encounteredKindStop'}->{$params{'-kind'}};
+	$k = $self->{'kindhash'}{$k};
+	$k = $self->{'kindhash'}{'?'} unless ($k);
+	(@a) = ("$k $t$hostname$pid" .
+		$self->{'encounteredMessageStop'}->{$params{'-kind'}});
+	$self->__print(@a);
+	exit $self->{'encounteredExit'}->{$params{'-kind'}};
+    }
+
     exit $params{'-exit'} if ($params{'-exit'});   # Aufhren, falls gesetzt
 
     if ($closeFile eq 'yes')
@@ -407,6 +461,7 @@ sub new
     bless $self, $class;
 }
 
+##################################################
 sub encountered
 {
     my $self = shift;
@@ -479,7 +534,8 @@ sub print
 	$prLog->print(%prLogs);
     }
 
-    if ($exit) {
+    if ($exit)
+    {
         main::cleanup($exit);
     }
 }
